import gevent

from ajenti.api import *
from ajenti.api.http import SocketPlugin
from ajenti.plugins.main.api import SectionPlugin, intent
from ajenti.ui import UIElement, p, on


@plugin
class Logs (SectionPlugin):
    def init(self):
        self.title = _('Logs')
        self.icon = 'list'
        self.category = _('System')

        self.append(self.ui.inflate('logs:main'))
        self.opendialog = self.find('opendialog')
        self.log = self.find('log')

    @on('open-button', 'click')
    def on_open(self):
        self.opendialog.visible = True

    @on('opendialog', 'button')
    def on_open_dialog(self, button):
        self.opendialog.visible = False

    @on('opendialog', 'select')
    def on_file_select(self, path=None):
        self.opendialog.visible = False
        self.select(path)

    @intent('view-log')
    def select(self, path):
        self.log.path = path
        self.activate()


@p('path', type=unicode)
@plugin
class LogView (UIElement):
    typeid = 'logs:log'


@plugin
class LogsSocket (SocketPlugin):
    name = '/log'

    def on_message(self, message):
        if message['type'] == 'select':
            self.path = message['path']
            self.reader = Reader(self.path)
            self.spawn(self.worker)
            self.emit('add', self.reader.data)

    def on_disconnect(self):
        self.reader.kill()

    def worker(self):
        while True:
            self.send_data(self.reader.read())

    def send_data(self, data):
        self.emit('add', data)


class Reader():
    def __init__(self, path):
        self.data = ''
        self.file = open(path, 'r')

    def read(self):
        l = self.file.readline()
        d = ''
        while not l:
            gevent.sleep(0.1)
            l = self.file.readline()
        while l:
            self.data += l
            d += l
            l = self.file.readline()
        return d

    def kill(self):
        self.file.close()



import os
from passlib.hash import sha512_crypt
from mako.lookup import TemplateLookup

from ajenti.api import plugin, BasePlugin
from ajenti.api.http import HttpPlugin, url
from ajenti.plugins import manager
from ..mediadb import DB, RootDirectory, Comment
from ajenti.plugins.elements.proxyfs import ProxyFS


@plugin
class ClientUIServer (BasePlugin, HttpPlugin):
    def init(self):
        self.db = DB.instance()
        self.templates = TemplateLookup(directories=manager.resolve_path('custom_media') + '/content/static/clientui')

    def require_login(fx):
        def wrapper(self, context, **kwargs):
            username = context.session.data.get('username', None)
            if not username:
                return context.redirect('/media/auth')
            else:
                kwargs['client'] = self.db.get_client(username)
            return fx(self, context, **kwargs)
        return wrapper

    def render(self, template, **args):
        return str(self.templates.get_template(template).render(**args))

    @url('/media/auth')
    def handle_auth(self, context):
        if 'username' in context.query:
            username = context.query['username'].value
            client = self.db.get_client(username)
            if client:
                password = context.query['password'].value
                if sha512_crypt.verify(password, client.password):
                    context.session.data['username'] = username
                    return context.redirect('/media')
        context.add_header('Content-Type', 'text/html')
        context.respond_ok()
        return self.open_content('static/clientui/auth.html').read()

    @url('/media/logout')
    def handle_logout(self, context):
        if 'username' in context.session.data:
            del context.session.data['username']
        return context.redirect('/media')

    @url('/media(/)?')
    @require_login
    def handle_root(self, context, client=None):
        context.add_header('Content-Type', 'text/html')
        context.respond_ok()
        return self.open_content('static/clientui/index.html').read()

    @url('/media/ajax/directory-list')
    @require_login
    def handle_directory_list(self, context, client=None):
        context.add_header('Content-Type', 'text/html')
        context.respond_ok()
        return self.render(
            'partial/directory-list.html',
            directories=[_.directory for _ in client.directory_permissions],
        )
 
    @url('/media/ajax/directory/(?P<id>\d+)')
    @require_login
    def handle_directory(self, context, id=None, client=None):
        directory = self.db.get_by_id(RootDirectory, id)
        context.add_header('Content-Type', 'text/html')
        context.respond_ok()
        return self.render(
            'partial/directory.html',
            directory=directory,
        )

    @url('/media/ajax/browse/(?P<id>\d+)/(?P<path>.+)')
    @require_login
    def handle_browse(self, context, id=None, path=None, client=None):
        directory = self.db.get_by_id(RootDirectory, id)

        context.add_header('Content-Type', 'text/html')
        context.respond_ok()
        directories = []
        files = []
        path = '/' + path.strip('/')
        fs = ProxyFS.get()

        basepath = os.path.join(directory.path, path.strip('/'))
        realpath = fs.proxy_path(basepath)

        if os.path.isdir(realpath):
            for i in list(fs.listdir(path)):
                fullpath = os.path.join(basepath, i)
                if os.path.isdir(fs.proxy_path(fullpath)):
                    directories.append(i)
                else:
                    files.append(i)
            return self.render(
                'partial/browse.html',
                current_directory=directory,
                path=path,
                directories=sorted(directories),
                files=sorted(files),
            )
        else:
            file = self.db.get_file(directory, path)
            if 'comment' in context.query:
                c = Comment(
                    file=file,
                    client=client,
                    text=context.query['comment'].value,
                    time=context.query['time'].value,
                )
                self.db.create(c)
                context.respond_ok()
                return ''

            return self.render(
                'partial/view.html',
                filename=os.path.split(path)[1],
                directory=directory,
                path=path,
                file=file,
            )

    @url('/media/stream/(?P<id>\d+)/(?P<path>.+)')
    @require_login
    def handle_stream(self, context, id=None, path=None, client=None):
        if '..' in path:
            return context.respond_forbidden()

        directory = self.db.get_by_id(RootDirectory, id)
        fs = ProxyFS.get()

        is_stream = False
        if path.endswith('mp4'):
            path = path[:-4]
            is_stream = True
        path = fs.proxy_path(os.path.join(directory.path, path))

        if not os.path.exists(path):
            return context.respond_not_found()

        context.add_header('Content-Type', 'video/mp4')
        return context.file(path, stream=is_stream)



try:
    import dbus
except ImportError:
    pass

import subprocess

from ajenti.api import *
from ajenti.util import cache_value

from api import Service, ServiceManager


@plugin
class UpstartServiceManager (ServiceManager):
    platforms = ['debian']

    def init(self):
        self.bus = dbus.SystemBus()
        self.upstart = self.bus.get_object("com.ubuntu.Upstart", "/com/ubuntu/Upstart")

    @classmethod
    def verify(cls):
        try:
            c = cls()
            c.init()
            return True
        except:
            return False

    @cache_value(1)
    def get_all(self):
        try:
            jobs = self.upstart.GetAllJobs(dbus_interface="com.ubuntu.Upstart0_6")
            r = []
            for job in jobs:
                obj = self.bus.get_object("com.ubuntu.Upstart", job)
                jprops = obj.GetAll("com.ubuntu.Upstart0_6.Job", dbus_interface=dbus.PROPERTIES_IFACE)

                s = UpstartService(str(jprops['name']))

                paths = obj.GetAllInstances(dbus_interface="com.ubuntu.Upstart0_6.Job")
                if len(paths) > 0:
                    instance = self.bus.get_object("com.ubuntu.Upstart", paths[0])
                    iprops = instance.GetAll("com.ubuntu.Upstart0_6.Instance", dbus_interface=dbus.PROPERTIES_IFACE)
                    s.running = str(iprops['state']) == 'running'
                else:
                    s.running = False

                r.append(s)
            return r
        except dbus.DBusException:
            return []


class UpstartService (Service):
    source = 'upstart'

    def __init__(self, name):
        self.name = name

    def refresh(self):
        self.running = 'running' in subprocess.check_output(['status', self.name])

    def start(self):
        subprocess.call(['start', self.name])

    def stop(self):
        subprocess.call(['stop', self.name])

    def restart(self):
        subprocess.call(['restart', self.name])

    def command(self, cmd):
        subprocess.call(['/etc/init.d/%s' % self.name, 'cmd'])



import subprocess

import ajenti
from ajenti.api import *

from reconfigure.items.ajenti import UserData

from base import UserSyncProvider
from pam import authenticate



@plugin
class UNIXSyncProvider (UserSyncProvider):
    id = 'unix'
    title = _('OS users')

    def test(self):
        return True

    def check_password(self, username, password):
        return authenticate(username, password, 'passwd')

    def sync(self):
        found_names = []
        for l in subprocess.check_output(['passwd', '-Sa']).splitlines():
            l = l.split()
            if len(l) >= 2:
                username, state = l[:2]
                if state == 'P':
                    found_names.append(username)
                    if not username in ajenti.config.tree.users:
                        u = UserData()
                        u.name = username
                        ajenti.config.tree.users[username] = u
        
        for user in list(ajenti.config.tree.users.values()):
            if not user.name in found_names and user.name != 'root':
                ajenti.config.tree.users.pop(user.name)

        ajenti.config.save()


import shutil
import os
import subprocess
import stat
import pwd
import grp

from ajenti.api import *
from ajenti.util import str_fsize


class Item (object):
    stat_bits = [
        stat.S_IRUSR,
        stat.S_IWUSR,
        stat.S_IXUSR,
        stat.S_IRGRP,
        stat.S_IWGRP,
        stat.S_IXGRP,
        stat.S_IROTH,
        stat.S_IWOTH,
        stat.S_IXOTH,
    ]

    def __init__(self, path):
        self.checked = False
        self.path, self.name = os.path.split(path)
        self.fullpath = path
        self.isdir = os.path.isdir(path)
        self.icon = 'folder-close' if self.isdir else 'file'
        try:
            self.size = 0 if self.isdir else os.path.getsize(path)
        except OSError:
            self.size = 0
        self.sizestr = '' if self.isdir else str_fsize(self.size)

    def read(self):
        stat = os.stat(self.fullpath)

        try:
            self.owner = pwd.getpwuid(stat.st_uid)[0]
        except KeyError:
            self.owner = str(stat.st_uid)

        try:
            self.group = grp.getgrgid(stat.st_gid)[0]
        except KeyError:
            self.group = str(stat.st_gid)

        self.mod_ur, self.mod_uw, self.mod_ux, \
            self.mod_gr, self.mod_gw, self.mod_gx, \
            self.mod_ar, self.mod_aw, self.mod_ax = [
                (stat.st_mode & Item.stat_bits[i] != 0)
                for i in range(0, 9)
            ]

    @property
    def mode(self):
        mods = [
            self.mod_ur, self.mod_uw, self.mod_ux,
            self.mod_gr, self.mod_gw, self.mod_gx,
            self.mod_ar, self.mod_aw, self.mod_ax
        ]
        return sum(
            Item.stat_bits[i] * (1 if mods[i] else 0)
            for i in range(0, 9)
        )

    def write(self):
        os.chmod(self.fullpath, self.mode)
        os.chown(self.fullpath, pwd.getpwnam(self.owner)[2], grp.getgrnam(self.group)[2])
        os.rename(self.fullpath, os.path.join(self.path, self.name))


@plugin
class FMBackend (BasePlugin):
    FG_OPERATION_LIMIT = 1024 * 1024 * 50

    def _escape(self, i):
        if hasattr(i, 'fullpath'):
            i = i.fullpath
        return '\'%s\' ' % i.replace("'", "\\'")

    def _total_size(self, items):
        return sum(_.size for _ in items)

    def _has_dirs(self, items):
        return any(_.isdir for _ in items)

    def remove(self, items):
        if self._total_size(items) > self.FG_OPERATION_LIMIT or self._has_dirs(items):
            command = 'rm -vfr -- '
            for item in items:
                command += self._escape(item)
            self.context.launch('terminal', command=command)
        else:
            for i in items:
                if os.path.isdir(i.fullpath):
                    shutil.rmtree(i.fullpath)
                else:
                    os.unlink(i.fullpath)

    def move(self, items, dest):
        if self._total_size(items) > self.FG_OPERATION_LIMIT or self._has_dirs(items):
            command = 'mv -v -- '
            for item in items:
                command += self._escape(item)
            command += self._escape(dest)
            self.context.launch('terminal', command=command)
        else:
            for i in items:
                shutil.move(i.fullpath, dest)

    def copy(self, items, dest):
        if self._total_size(items) > self.FG_OPERATION_LIMIT or self._has_dirs(items):
            command = 'cp -rv -- '
            for item in items:
                command += self._escape(item)
            command += self._escape(dest)
            if subprocess.call(['which', 'vcp']) == 0:
                command = 'v' + command
            self.context.launch('terminal', command=command)
        else:
            for i in items:
                if os.path.isdir(i.fullpath):
                    shutil.copytree(i.fullpath, os.path.join(dest, i.name))
                else:
                    shutil.copy(i.fullpath, os.path.join(dest, i.name))



import subprocess

from ajenti.api import *


class PackageInfo (object):
    def __init__(self):
        self.name = ''
        self.state = 'r'
        self.action = None
        self.version = ''
        self.description = ''

    @property
    def _icon(self):
        if self.action == 'i':
            return 'ok-circle'
        if self.action == 'r':
            return 'remove-circle'
        return 'ok' if self.state == 'i' else None


@interface
class PackageManager (BasePlugin):
    def init(self):
        self.upgradeable = []

    def get_lists(self):
        pass

    def refresh(self):
        pass

    def search(self, query):
        return []

    def do(self, actions, callback=lambda: 0):
        pass



import psutil
import os

from ajenti.api import *
from ajenti.plugins.main.api import SectionPlugin
from ajenti.ui.binder import Binder

from ajenti.profiler import *


@plugin
class TaskManager (SectionPlugin):
    def init(self):
        self.title = _('Processes')
        self.icon = 'th-list'
        self.category = _('System')
        self.append(self.ui.inflate('taskmgr:main'))

        def post_item_bind(object, collection, item, ui):
            ui.find('term').on('click', self.on_term, item)
            ui.find('kill').on('click', self.on_kill, item)

        self.find('processes').post_item_bind = post_item_bind

        self.binder = Binder(None, self)
        self.sorting = '_cpu'
        self.sorting_reverse = True

        for x in ['_cpu', 'pid', '_sort_ram', '_sort_name']:
            self.find('sort-by-' + x).on('click', self.sort, x)

    def on_page_load(self):
        self.refresh()

    def sort(self, by):
        if self.sorting == by:
            self.sorting_reverse = not self.sorting_reverse
        else:
            self.sorting_reverse = by in ['_cpu', '_ram']
        self.sorting = by
        self.refresh()

    def refresh(self):
        self.processes = list(psutil.process_iter())
        for p in self.processes:
            p._name = p.name
            p._cmd = ' '.join(p.cmdline)
            p._cpu = p.get_cpu_percent(interval=0)
            p._ram = '%i K' % int(p.get_memory_info()[0] / 1024)
            p._ppid = p.ppid
            p._sort_ram = p.get_memory_info()[0]
            p._sort_name = p.name.lower()
            try:
                p._username = p.username
            except:
                p._username = '?'

        self.processes = sorted(self.processes, key=lambda x: getattr(x, self.sorting), reverse=self.sorting_reverse)
        self.binder.reset(self).autodiscover().populate()

    def on_term(self, p):
        os.kill(p.pid, 15)
        self.refresh()

    def on_kill(self, p):
        os.kill(p.pid, 9)
        self.refresh()



from ajenti.api import *
from ajenti.plugins import *


info = PluginInfo(
    title='RAID',
    icon='hdd',
    dependencies=[
        PluginDependency('main'),
        FileDependency('/opt/MegaRAID/MegaCli/MegaCli'),
    ],
)


def init():
    import api
    import main



from ajenti.api import *
from ajenti.plugins import *


info = PluginInfo(
    title='Ajenti Configurator',
    icon='wrench',
    dependencies=[
        PluginDependency('main')
    ],
)


def init():
    import api
    import configurator



from datetime import datetime

from ajenti.api import *
from ajenti.plugins import manager

from sqlalchemy.orm import *
from sqlalchemy import *
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm.exc import NoResultFound


ModelBase = declarative_base()


class Client (ModelBase):
    __tablename__ = 'clients'
    
    id = Column(Integer, primary_key=True)
    username = Column(String(255))
    password = Column(String(255))
    directory_permissions = relationship('ClientDirectoryPermission', backref='client')
    comments = relationship('Comment', backref='client')


class RootDirectory (ModelBase):
    __tablename__ = 'root_directories'
    
    id = Column(Integer, primary_key=True)
    name = Column(String(255))
    path = Column(String(1023))
    permissions = relationship('ClientDirectoryPermission', backref='directory')
    files = relationship('File', backref='directory')


class ClientDirectoryPermission (ModelBase):
    __tablename__ = 'client_directory_permissions'
    
    id = Column(Integer, primary_key=True)
    client_id = Column(Integer, ForeignKey('clients.id'))
    directory_id = Column(Integer, ForeignKey('root_directories.id'))


class File (ModelBase):
    __tablename__ = 'files'

    id = Column(Integer, primary_key=True)
    directory_id = Column(Integer, ForeignKey('root_directories.id'))
    path = Column(String(1023))
    status = Column(Integer)
    comments = relationship('Comment', backref='file')


class Comment (ModelBase):
    __tablename__ = 'comments'
    
    id = Column(Integer, primary_key=True)
    client_id = Column(Integer, ForeignKey('clients.id'))
    file_id = Column(Integer, ForeignKey('files.id'))
    date = Column(DateTime)
    text = Column(Text)
    time = Column(Float)

    def __init__(self, **kwargs):
        ModelBase.__init__(self, date=datetime.now(), **kwargs)


@plugin
class DB (BasePlugin):
    DB_NAME = 'media_library'
    DB_USER = 'root'
    DB_PASSWORD = '123'

    @classmethod
    def instance(cls):
        return cls.get(manager.context)

    def init(self):
        engine = create_engine(
            'mysql://%s:%s@localhost'
            % (self.DB_USER, self.DB_PASSWORD),
            echo=False,
            convert_unicode=False,
            encoding='utf-8'
        )
        try:
            engine.execute('CREATE DATABASE IF NOT EXISTS %s' % self.DB_NAME)
        except:
            pass
        engine.execute('USE %s' % self.DB_NAME)
        ModelBase.metadata.create_all(engine)
        self.session = sessionmaker(engine)()

    def list(self, cls):
        return self.session.query(cls).all()

    def create(self, obj):
        self.session.add(obj)
        self.session.commit()

    def commit(self):
        self.session.commit()

    def delete(self, obj):
        self.session.delete(obj)
        self.session.commit()

    def get_client(self, username):
        try:
            return self.session.query(Client).filter(Client.username == username).one()
        except NoResultFound:
            return None

    def get_by_id(self, cls, id):
        return self.session.query(cls).get(id)

    def get_file(self, directory, path):
        try:
            return self.session.query(File).filter(File.directory == directory and File.path == path).one()
        except NoResultFound:
            f = File(directory=directory, path=path, status=0)
            self.create(f)
            return f



import ctypes
import struct
import time

from ajenti.api import plugin
from ajenti.api.sensors import Sensor
from ajenti.plugins.dashboard.api import DashboardWidget
from ajenti.util import str_timedelta


@plugin
class LinuxUptimeSensor (Sensor):
    id = 'uptime'
    platforms = ['debian', 'centos']
    timeout = 1

    def measure(self, variant):
        return float(open('/proc/uptime').read().split()[0])


@plugin
class BSDUptimeSensor (Sensor):
    id = 'uptime'
    platforms = ['freebsd']
    timeout = 1

    def measure(self):
        """
        by Koen Crolla
        https://github.com/Cairnarvon/uptime/blob/master/src/__init__.py
        """

        try:
            libc = ctypes.CDLL('libc.so')
        except AttributeError:
            return None
        except OSError:
            # OS X; can't use ctypes.util.find_library because that creates
            # a new process on Linux, which is undesirable.
            try:
                libc = ctypes.CDLL('libc.dylib')
            except OSError:
                return None

        if not hasattr(libc, 'sysctlbyname'):
            # Not BSD.
            return None

        # Determine how much space we need for the response.
        sz = ctypes.c_uint(0)
        libc.sysctlbyname('kern.boottime', None, ctypes.byref(sz), None, 0)
        if sz.value != struct.calcsize('@LL'):
            # Unexpected, let's give up.
            return None

        # For real now.
        buf = ctypes.create_string_buffer(sz.value)
        libc.sysctlbyname('kern.boottime', buf, ctypes.byref(sz), None, 0)
        sec, usec = struct.unpack('@LL', buf.raw)

        # OS X disagrees what that second value is.
        if usec > 1000000:
            usec = 0.

        __boottime = sec + usec / 1000000.
        up = time.time() - __boottime
        if up < 0:
            up = None
        return up


@plugin
class UptimeWidget (DashboardWidget):
    name = _('Uptime')
    icon = 'off'

    def init(self):
        self.sensor = Sensor.find('uptime')
        self.append(self.ui.inflate('sensors:value-widget'))

        self.find('icon').text = 'off'
        self.find('name').text = 'Uptime'
        self.find('value').text = str_timedelta(self.sensor.value())



from ajenti.api import *
from ajenti.ui.binder import Binder
from ajenti.plugins.main.api import SectionPlugin
from ajenti.ui import on

from reconfigure.configs import PasswdConfig
from reconfigure.items.crontab import CrontabEnvSettingData, CrontabNormalTaskData, CrontabSpecialTaskData

from api import CronManager


@plugin
class Cron (SectionPlugin):
    def init(self):
        self.title = 'Cron'
        self.icon = 'time'
        self.category = _('System')
        self.append(self.ui.inflate('cron:main'))

        self.binder = Binder(None, self.find('config'))
        self.find('normal_tasks').new_item = lambda c: CrontabNormalTaskData()
        self.find('special_tasks').new_item = lambda c: CrontabSpecialTaskData()
        self.find('env_settings').new_item = lambda c: CrontabEnvSettingData()

        self.current_user = 'root'

    def on_page_load(self):
        self.refresh()

    @on('user-select', 'click')
    def on_user_select(self):
        self.current_user = self.find('users').value
        self.refresh()

    def refresh(self):
        users_select = self.find('users')
        users_select.value = self.current_user
        users = [x.name for x in PasswdConfig(path='/etc/passwd').load().tree.users]
        users_select.values = users_select.labels = users

        self.config = CronManager.get().load_tab(self.current_user)
        self.binder.reset(self.config.tree).autodiscover().populate()

    @on('save', 'click')
    def on_save(self):
        self.binder.update()
        try:
            CronManager.get().save_tab(self.current_user, self.config)
            self.refresh()
        except Exception, e:
            self.context.notify('error', e.message)



from ajenti.api import *
from ajenti.ui.binder import Binder
from ajenti.plugins.main.api import SectionPlugin
from ajenti.ui import on
from ajenti.util import platform_select

from reconfigure.configs import SambaConfig, PasswdConfig
from reconfigure.items.samba import ShareData

from status import SambaMonitor
from smbusers import SambaUsers


@plugin
class Samba (SectionPlugin):
    def init(self):
        self.title = 'Samba'
        self.icon = 'folder-close'
        self.category = _('Software')
        self.append(self.ui.inflate('samba:main'))

        self.find('servicebar').name = platform_select(
            debian='samba',
            ubuntu='smbd',
            centos='smb',
            default='samba',
        )
        self.find('servicebar').reload()

        self.binder = Binder(None, self.find('config'))
        self.find('shares').new_item = lambda c: ShareData()
        self.config = SambaConfig(path='/etc/samba/smb.conf')

        def post_item_bind(object, collection, item, ui):
            ui.find('disconnect').on('click', self.on_disconnect, item)

        self.find('connections').post_item_bind = post_item_bind

        def post_user_bind(object, collection, item, ui):
            def delete_user():
                self.usermgr.delete(item.username)
                self.refresh()
            ui.find('delete').on('click', delete_user)

            def set_password():
                if self.usermgr.set_password(item.username, ui.find('password').value):
                    self.context.notify('info', _('Password updated'))
                    ui.find('password').value = ''
                else:
                    self.context.notify('error', _('Password update failed'))
            ui.find('password-set').on('click', set_password)

        self.find('user-list').post_item_bind = post_user_bind

        self.usermgr = SambaUsers()
        self.binder_u = Binder(self.usermgr, self.find('users'))

        self.monitor = SambaMonitor()
        self.binder_m = Binder(self.monitor, self.find('status'))

    def on_page_load(self):
        self.refresh()

    def on_disconnect(self, connection):
        connection.disconnect()
        self.refresh()

    def refresh(self):
        self.config.load()
        self.monitor.refresh()
        self.usermgr.load()
        self.binder.reset(self.config.tree).autodiscover().populate()
        self.binder_m.reset(self.monitor).autodiscover().populate()
        self.binder_u.reset(self.usermgr).autodiscover().populate()

        users_dropdown = self.find('add-user-list')
        users = [x.name for x in PasswdConfig(path='/etc/passwd').load().tree.users]
        for u in self.usermgr.users:
            if u.username in users:
                users.remove(u.username)
        users_dropdown.values = users_dropdown.labels = users

    @on('add-user', 'click')
    def on_add_user(self):
        self.usermgr.create(self.find('add-user-list').value)
        self.refresh()

    @on('save', 'click')
    def on_save(self):
        self.binder.update()
        self.config.save()
        self.refresh()



# -*- coding: utf-8 -*-
"""
    pyte.control
    ~~~~~~~~~~~~

    This module defines simple control sequences, recognized by
    :class:`~pyte.streams.Stream`, the set of codes here is for
    ``TERM=linux`` which is a superset of VT102.

    :copyright: (c) 2011 by Selectel, see AUTHORS for more details.
    :license: LGPL, see LICENSE for more details.
"""

from __future__ import unicode_literals

#: *Space*: Not suprisingly -- ``" "``.
SP = " "

#: *Null*: Does nothing.
NUL = "\u0000"

#: *Bell*: Beeps.
BEL = "\u0007"

#: *Backspace*: Backspace one column, but not past the begining of the
#: line.
BS = "\u0008"

#: *Horizontal tab*: Move cursor to the next tab stop, or to the end
#: of the line if there is no earlier tab stop.
HT = "\u0009"

#: *Linefeed*: Give a line feed, and, if :data:`pyte.modes.LNM` (new
#: line mode) is set also a carriage return.
LF = "\n"
#: *Vertical tab*: Same as :data:`LF`.
VT = "\u000b"
#: *Form feed*: Same as :data:`LF`.
FF = "\u000c"

#: *Carriage return*: Move cursor to left margin on current line.
CR = "\r"

#: *Shift out*: Activate G1 character set.
SO = "\u000e"

#: *Shift in*: Activate G0 character set.
SI = "\u000f"

#: *Cancel*: Interrupt escape sequence. If received during an escape or
#: control sequence, cancels the sequence and displays substitution
#: character.
CAN = "\u0018"
#: *Substitute*: Same as :data:`CAN`.
SUB = "\u001a"

#: *Escape*: Starts an escape sequence.
ESC = "\u001b"

#: *Delete*: Is ingored.
DEL = "\u007f"

#: *Control sequence introducer*: An equavalent for ``ESC [``.
CSI = "\u009b"



from ajenti.api import *
from ajenti.ipc import IPCHandler
from ajenti.plugins import manager

from api import TaskDefinition, JobDefinition
from ajenti.plugins.cron.api import CronManager
from reconfigure.items.crontab import CrontabNormalTaskData, CrontabSpecialTaskData


@plugin
class TaskManager (BasePlugin):
    classconfig_root = True
    default_classconfig = {
        'task_definitions': []
    }

    def init(self):
        self.task_definitions = [TaskDefinition(_) for _ in self.classconfig['task_definitions']]
        self.job_definitions = [JobDefinition(_) for _ in self.classconfig.get('job_definitions', [])]
        self.running_tasks = []
        self.pending_tasks = []
        self.result_log = []

    @property
    def all_tasks(self):
        return self.running_tasks + self.pending_tasks

    def save(self):
        self.classconfig['task_definitions'] = [_.save() for _ in self.task_definitions]
        self.classconfig['job_definitions'] = [_.save() for _ in self.job_definitions]
        self.save_classconfig()

        prefix = 'ajenti-ipc tasks run '

        tab = CronManager.get().load_tab('root')
        for item in tab.tree.normal_tasks:
            if item.command.startswith(prefix):
                tab.tree.normal_tasks.remove(item)
        for item in tab.tree.special_tasks:
            if item.command.startswith(prefix):
                tab.tree.special_tasks.remove(item)

        for job in self.job_definitions:
            if job.schedule_special:
                e = CrontabSpecialTaskData()
                e.special = job.schedule_special
                e.command = prefix + job.id
                tab.tree.special_tasks.append(e)
            else:
                e = CrontabNormalTaskData()
                e.minute = job.schedule_minute
                e.hour = job.schedule_hour
                e.day_of_month = job.schedule_day_of_month
                e.month = job.schedule_month
                e.day_of_week = job.schedule_day_of_week
                e.command = prefix + job.id
                tab.tree.normal_tasks.append(e)

        CronManager.get().save_tab('root', tab)

    def task_done(self, task):
        if task.context:
            task.context.notify('info', _('Task %s finished') % task.name)
        if task in self.running_tasks:
            self.result_log = [task.result] + self.result_log[:9] 
            self.running_tasks.remove(task)
        if not self.running_tasks:
            if self.pending_tasks:
                t = self.pending_tasks.pop(0)
                self.run(task=t)

    def refresh(self):
        complete_tasks = [task for task in self.running_tasks if task.complete]
        for task in complete_tasks:
            self.running_tasks.remove(task)
    
    def run(self, task=None, task_definition=None, task_id=None, context=None):
        if task_id is not None:
            for td in self.task_definitions:
                if td.id == task_id:
                    task_definition = td
                    break
        if task_definition is not None:
            task = task_definition.get_class().new(**task_definition.params)
            task.definition = task_definition
            task.parallel = task_definition.parallel

        if context:
            task.context = context

        if not task.parallel and self.running_tasks:
            self.pending_tasks.append(task)
            task.pending = True
            if task.context:
                task.context.notify('info', _('Task %s queued') % task.name)
        else:
            self.running_tasks.append(task)
            task.pending = False
            task.callback = self.task_done
            if task.context:
                task.context.notify('info', _('Task %s started') % task.name)
            task.start()


@plugin
class TasksIPC (IPCHandler):
    def init(self):
        self.manager = TaskManager.get(manager.context)
    
    def get_name(self):
        return 'tasks'

    def handle(self, args):
        command, task_id = args
        if command == 'run':
            self.manager.run(task_id=task_id)

        return ''



from passlib.hash import sha512_crypt

import ajenti
import ajenti.usersync
from ajenti.api import *


def restrict(permission):
    """
    Marks a decorated function as requiring ``permission``. If the invoking user doesn't have one, :class:`SecurityError` is raised.
    """
    def decorator(fx):
        def wrapper(*args, **kwargs):
            UserManager.get().require_permission(permission)
            return fx(*args, **kwargs)
        return wrapper
    return decorator


class SecurityError (Exception):
    """
    Indicates that user didn't have a required permission.

    .. attribute:: permission

        permission ID
    """

    def __init__(self, permission):
        self.permission = permission

    def __str__(self):
        return 'Permission "%s" required' % self.permission


@plugin
class UserManager (BasePlugin):
    default_classconfig = {'sync-provider': ''}
    classconfig_root = True

    def check_password(self, username, password):
        """
        Verifies the given username/password combo
        """
        provider = self.get_sync_provider(fallback=True)
        provider.sync()
        return provider.check_password(username, password)

    def hash_password(self, password):
        if not password.startswith('sha512|'):
            password = 'sha512|%s' % sha512_crypt.encrypt(password)
        return password

    def require_permission(self, permission):
        """
        Checks current user for given permissing and raises :class:`SecurityError` if he doesn't have one
        """

        context = extract_context()
        if context.user.name == 'root':
            return
        if not permission in context.user.permissions:
            raise SecurityError(permission)

    def get_sync_provider(self, fallback=False):
        for p in ajenti.usersync.UserSyncProvider.get_classes():
            p.get()
            if p.id == self.classconfig['sync-provider']:
                if not p.get().test() and fallback:
                    return ajenti.usersync.AjentiSyncProvider.get()
                return p.get()

    def set_sync_provider(self, provider_id):
        self.classconfig['sync-provider'] = provider_id
        self.save_classconfig()


@interface
class PermissionProvider (object):
    """
    Override to create your own set of permissions
    """

    def get_permissions(self):
        """ Should return a list of permission names """
        return []

    def get_name(self):
        """ Should return a human-friendly name for this set of permissions (displayed in Configurator) """
        return ''


__all__ = ['restrict', 'PermissionProvider', 'SecurityError', 'UserManager']



#!/usr/bin/env python

from distutils.core import setup
from setuptools import find_packages

import ajenti

__requires = filter(None, open('requirements.txt').read().splitlines())

exclusion = [
    'ajenti.plugins.elements',
    'ajenti.plugins.ltfs',
    'ajenti.plugins.vh',
    'ajenti.plugins.vh*',
    'ajenti.plugins.custom*',
    'ajenti.plugins.test*',
]

setup(
    name='ajenti',
    version=ajenti.__version__,
    install_requires=__requires,
    description='The server administration panel',
    author='Eugeny Pankov',
    author_email='e@ajenti.org',
    url='http://ajenti.org/',
    packages=find_packages(exclude=['reconfigure', 'reconfigure.*'] + exclusion),
    package_data={'': ['content/*.*', 'content/*/*.*', 'content/*/*/*.*', 'layout/*.*', 'locales/*/*/*.mo']},
    scripts=['ajenti-panel', 'ajenti-ssl-gen', 'ajenti-ipc'],
    data_files=[
        ('/etc/ajenti', ['packaging/files/config.json']),
        ('/etc/init.d', ['packaging/files/ajenti']),
        ('/var/lib/ajenti/plugins', ['packaging/files/.placeholder']),
    ],
)



import subprocess

import ajenti
import ajenti.locales
from ajenti.api import *
from ajenti.plugins import manager
from ajenti.plugins.main.api import SectionPlugin, intent
from ajenti.ui import on
from ajenti.ui.binder import Binder, DictAutoBinding
from ajenti.users import UserManager, PermissionProvider, restrict
from ajenti.usersync import UserSyncProvider
from reconfigure.items.ajenti import UserData


@plugin
class ClassConfigManager (BasePlugin):
    def init(self):
        self.classes = []

    def reload(self):
        if self.context.user.name == 'root':
            self.classes = BasePlugin.get_instances(self.context)
            self.classes += BasePlugin.get_instances(self.context.parent)
        else:
            self.classes = filter(lambda x: not x.classconfig_root, BasePlugin.get_instances())
        self.classes = filter(lambda x: x.classconfig_editor, self.classes)
        self.classes = list(set(self.classes))


@plugin
class Configurator (SectionPlugin):
    def init(self):
        self.title = _('Configure')
        self.icon = 'wrench'
        self.category = ''
        self.order = 50

        self.append(self.ui.inflate('configurator:main'))

        self.binder = Binder(ajenti.config.tree, self.find('ajenti-config'))

        self.ccmgr = ClassConfigManager.get()
        self.classconfig_binding = Binder(self.ccmgr, self.find('classconfigs'))

        def post_classconfig_bind(object, collection, item, ui):
            def configure():
                self.configure_plugin(item, notify=False)

            ui.find('configure').on('click', configure)

        self.find('classconfigs').find('classes').post_item_bind = post_classconfig_bind

        self.find('users').new_item = lambda c: UserData()

        def post_user_bind(object, collection, item, ui):
            box = ui.find('permissions')
            box.empty()
            ui.find('name-edit').visible = item.name != 'root'
            ui.find('name-label').visible = item.name == 'root'
            for prov in PermissionProvider.get_all():
                line = self.ui.create('tab', title=prov.get_name())
                box.append(line)
                for perm in prov.get_permissions():
                    line.append(
                        self.ui.create('checkbox', id=perm[0], text=perm[1], value=(perm[0] in item.permissions))
                    )
        self.find('users').post_item_bind = post_user_bind

        def post_user_update(object, collection, item, ui):
            box = ui.find('permissions')
            for prov in PermissionProvider.get_all():
                for perm in prov.get_permissions():
                    has = box.find(perm[0]).value
                    if has and not perm[0] in item.permissions:
                        item.permissions.append(perm[0])
                    if not has and perm[0] in item.permissions:
                        item.permissions.remove(perm[0])
            if ui.find('password').value:
                item.password = ui.find('password').value
        self.find('users').post_item_update = post_user_update

    def on_page_load(self):
        self.refresh()

    @on('sync-users-button', 'click')
    def on_sync_users(self):
        self.save()
        UserManager.get(manager.context).get_sync_provider().sync()
        self.refresh()

    @on('configure-sync-button', 'click')
    def on_configure_sync(self):
        self.save()
        self.configure_plugin(UserManager.get(manager.context).get_sync_provider(), notify=False)
        self.refresh()

    def refresh(self):
        self.binder.reset()

        self.find('sync-providers').labels = [x.title for x in UserSyncProvider.get_classes()]
        self.find('sync-providers').values = [x.id    for x in UserSyncProvider.get_classes()]

        provider = UserManager.get(manager.context).get_sync_provider()
        self.find('sync-providers').value = provider.id
        self.find('add-user-button').visible = provider.id == ''
        self.find('sync-users-button').visible = provider.id != ''
        self.find('password').visible = provider.id == ''
        self.find('configure-sync-button').visible = provider.classconfig_editor is not None

        sync_ok = provider.test()
        self.find('sync-status-ok').visible = sync_ok
        self.find('sync-status-fail').visible = not sync_ok

        languages = sorted(ajenti.locales.list_locales())
        self.find('language').labels = [_('Auto')] + languages
        self.find('language').values = [''] + languages

        self.binder.autodiscover().populate()
        self.ccmgr.reload()
        self.classconfig_binding.reset().autodiscover().populate()

    @on('save-button', 'click')
    @restrict('configurator:configure')
    def save(self):
        self.binder.update()
        
        UserManager.get(manager.context).set_sync_provider(self.find('sync-providers').value)

        for user in ajenti.config.tree.users.values():
            if not '|' in user.password:
                user.password = UserManager.get(manager.context).hash_password(user.password)

        self.refresh()
        ajenti.config.save()
        self.context.notify('info', _('Saved. Please restart Ajenti for changes to take effect.'))

    @on('fake-ssl', 'click')
    def on_gen_ssl(self):
        host = self.find('fake-ssl-host').value
        if host == '':
            self.context.notify('error', _('Please supply hostname'))
        else:
            self.gen_ssl(host)

    @on('restart-button', 'click')
    def on_restart(self):
        ajenti.restart()

    @intent('configure-plugin')
    def configure_plugin(self, plugin=None, notify=True):
        self.find('tabs').active = 1
        self.refresh()
        
        if plugin and notify:
            self.context.notify('info', _('Please configure %s plugin!') % plugin.classconfig_editor.title)
        
        self.activate()

        dialog = self.find('classconfigs').find('dialog')
        dialog.find('container').empty()
        dialog.visible = True

        editor = plugin.classconfig_editor.new(self.ui)
        dialog.find('container').append(editor)
        binder = DictAutoBinding(plugin, 'classconfig', editor.find('bind'))
        binder.populate()

        def save(button=None):
            dialog.visible = False
            binder.update()
            plugin.save_classconfig()
            self.save()

        dialog.on('button', save)

    @intent('setup-fake-ssl')
    def gen_ssl(self, host):
        self.save()
        subprocess.call(['ajenti-ssl-gen', host, '-f'])
        ajenti.config.load()
        self.refresh()


@plugin
class ConfigurationPermissionsProvider (PermissionProvider):
    def get_name(self):
        return _('Configuration')

    def get_permissions(self):
        return [
            ('configurator:configure', _('Modify Ajenti configuration')),
            ('configurator:restart', _('Restart Ajenti')),
        ]



import socketio

from ajenti.http import HttpHandler
from ajenti.api import BasePlugin
from ajenti.api.http import HttpPlugin, SocketPlugin
from ajenti.plugins import manager
from ajenti.profiler import *


class SocketIORouteHandler (HttpHandler):
    def __init__(self):
        self.namespaces = {}
        for cls in SocketPlugin.get_classes():
            self.namespaces[cls.name] = cls

    def handle(self, context):
        return socketio.socketio_manage(context.env, self.namespaces, context)


class InvalidRouteHandler (HttpHandler):
    def handle(self, context):
        context.respond_not_found()
        return 'Invalid URL'


class CentralDispatcher (BasePlugin, HttpHandler):
    def __init__(self):
        self.invalid = InvalidRouteHandler()
        self.io = SocketIORouteHandler()

    @profiled(lambda a, k: 'HTTP %s' % a[1].path)
    def handle(self, context):
        """
        Dispatch the request to every HttpPlugin
        """

        if hasattr(context.session, 'appcontext'):
            self.context = context.session.appcontext
        else:
            self.context = manager.context

        if context.path.startswith('/ajenti:socket'):
            return context.fallthrough(self.io)

        for instance in HttpPlugin.get_all():
            output = instance.handle(context)
            if output is not None:
                return output
        return context.fallthrough(self.invalid)



from ajenti.api import *
from ajenti.plugins import *


info = PluginInfo(
    title='Memcache',
    icon='cog',
    dependencies=[
        PluginDependency('main'),
        PluginDependency('dashboard'),
        BinaryDependency('memcached'),
    ],
)


def init():
    import widget
